--
-- FrontloaderAttacher
-- Specialization for new frontloader joint
-- This joint (attach and detach actions) differs from the original one.
-- So a new specialization is necessary.
--
-- SFM-Modding
-- @author  Manuel Leithner
-- @date  28/12/09
--

FrontloaderAttacher = {};

function FrontloaderAttacher.prerequisitesPresent(specializations)
	Vehicle.registerJointType("attachableFrontloader");
    return SpecializationUtil.hasSpecialization(Motorized, specializations);
end;

function FrontloaderAttacher:load(xmlFile)

	self.getFrontloaderInRange = FrontloaderAttacher.getFrontloaderInRange;
	self.getAttachedTrailersFillLevelAndCapacity = FrontloaderAttacher.getAttachedTrailersFillLevelAndCapacity;

	local frontloaderAttachingPoint = {};
	frontloaderAttachingPoint.joint = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.frontloaderAttachingPoint#index"));
	frontloaderAttachingPoint.index = 0;
	self.frontloader = {};
	self.frontloader.node = frontloaderAttachingPoint;	
	self.attachedFrontloader = nil;
	self.frontloaderInRange = nil;	
	
	local jointTypeStr = getXMLString(xmlFile, "vehicle.frontloaderAttachingPoint#jointType")
	if jointTypeStr ~= nil then		
		self.frontloader.jointType = Vehicle.jointTypeNameToInt[jointTypeStr];
	end;
	
	self.errorMessageTime = 0;	
end;

function FrontloaderAttacher:delete()
end;

function FrontloaderAttacher:mouseEvent(posX, posY, isDown, isUp, button)
	if self.attachedFrontloader ~= nil then
		self.attachedFrontloader:mouseEvent(posX, posY, isDown, isUp, button);
	end;
end;

function FrontloaderAttacher:keyEvent(unicode, sym, modifier, isDown)
	if self.attachedFrontloader ~= nil then
		self.attachedFrontloader:keyEvent(unicode, sym, modifier, isDown);
	end;
end;


function FrontloaderAttacher:update(dt)
		
	if self.isMotorStarted then	
		local localFrontloader = self.frontloader.node;
		if InputBinding.hasEvent(InputBinding.GTA_AttachFrontloader) and not self.isHired then
			if self.attachedFrontloader ~= nil then
				local dettachingAllowed = true;				
				if self.attachedFrontloader.isDetachAllowed ~= nil then
					dettachingAllowed = self.attachedFrontloader.isDetachAllowed;
				end;				
				if dettachingAllowed then
					AITractor.removeToolTrigger(self, self.attachedFrontloader);
					if self.attachedFrontloader.attachedTool ~= nil then
						AITractor.removeToolTrigger(self, self.attachedFrontloader.attachedTool);
					end;
					self.attachedFrontloader.attacherVehicle = nil;
					self.attachedFrontloader:onDetach();					
					removeJoint(localFrontloader.index);
					localFrontloader.index = 0;
					self.attachedFrontloader = nil;
				else
					self.errorMessageTime = 70;
				end;
			else
				if self.frontloaderInRange ~= nil then
					local implement = {};
					self.attachedFrontloader = self.frontloaderInRange;
					self.attachedFrontloader:onAttach(self);
					AITractor.addToolTrigger(self, self.attachedFrontloader);
					if self.attachedFrontloader.attachedTool ~= nil then
						AITractor.addToolTrigger(self, self.attachedFrontloader.attachedTool);
					end;
					local constr = JointConstructor:new();
					constr:setActors(self.components[1].node, self.frontloaderInRange.components[1].node);
					constr:setJointTransforms(localFrontloader.joint, self.frontloaderInRange.attacherJoint.node);
					for i=1, 3 do		                    
						constr:setTranslationLimit(i-1, true, 0, 0);
						constr:setRotationLimit(i-1, 0, 0);
					end;
					localFrontloader.index = constr:finalize();
				end;
			end;
		end;	
			
		-- frontloader handling
		if self.attachedFrontloader == nil then
			-- get available frontloader
			self.frontloaderInRange = self:getFrontloaderInRange();
		else
			self.attachedFrontloader.isActive = true;
			self.attachedFrontloader.isSelected = true;
		end;
	end;			
end;


function FrontloaderAttacher:draw()
	if self:getIsActive() then
		if self.attachedFrontloader ~= nil then
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("deattach_Frontloader"), self.typeDesc), InputBinding.GTA_AttachFrontloader);
		else
			if self.frontloaderInRange ~= nil then
				-- print help menu if frontloader is available
				g_currentMission.hudAttachmentOverlay:render();
				g_currentMission:addHelpButtonText(string.format(g_i18n:getText("attach_Frontloader"), self.typeDesc), InputBinding.GTA_AttachFrontloader);
			end;
		end;
	end;
	if self.attachedFrontloader ~= nil then
		self.attachedFrontloader:draw();
		
		if self.attachedFrontloader.noDetachAllowedMsg ~= nil then
			if self.errorMessageTime ~= 0 then
				g_currentMission:addWarning(self.attachedFrontloader.noDetachAllowedMsg, 0, 0.03);
				self.errorMessageTime = self.errorMessageTime - 1;
			end;
		end;
	end;
end;


function FrontloaderAttacher:getFrontloaderInRange()
    local nearestObject = nil;
	local frontloaderCopy = self.frontloader.node;	
    if g_currentMission.currentVehicle ~= nil and g_currentMission.currentVehicle == self then
	
        local nearestDistance = 1.0;
		local px, py, pz = getWorldTranslation(frontloaderCopy.joint);
		
		for i=1, table.getn(g_currentMission.attachables) do
			if g_currentMission.attachables[i].attacherJoint.node ~= nil then	
					
				local vx, vy, vz = getWorldTranslation(g_currentMission.attachables[i].attacherJoint.node);	
				local distance = Utils.vector3Length(px-vx, py-vy, pz-vz);
				if distance < nearestDistance then
					if g_currentMission.attachables[i].attacherVehicle == nil then
						
						nearestObject = g_currentMission.attachables[i];
						nearestDistance = distance;
					end;
				end;
			end;
		end;
		if nearestObject ~= nil then
			if self.frontloader.jointType ~= nearestObject.attacherJoint.jointType then
				nearestObject = nil;
			end;
		end;
    end;
    return nearestObject;
end;

function FrontloaderAttacher:getAttachedTrailersFillLevelAndCapacity()
    local fillLevel = 0;
    local capacity = 0;
    local hasTrailer = false;
    for k, implement in pairs(self.attachedImplements) do
        local object = implement.object;
        if object.fillLevel ~= nil and object.capacity ~= nil then
            fillLevel = fillLevel + object.fillLevel;
            capacity = capacity + object.capacity;
            hasTrailer = true;
        end;
        local f, c = implement.object:getAttachedTrailersFillLevelAndCapacity();
        if f ~= nil and c ~= nil then
            fillLevel = fillLevel + f;
            capacity = capacity + c;
            hasTrailer = true;
        end;
    end;
	if self.attachedFrontloader ~= nil then
		local f = self.attachedFrontloader.fillLevel;
		local c = self.attachedFrontloader.capacity;
		
        if f ~= nil and c ~= nil then
            fillLevel = fillLevel + f;
            capacity = capacity + c;
            hasTrailer = true;
        end;

		if self.attachedFrontloader.getAttachedTrailersFillLevelAndCapacity ~= nil then
			f,c = self.attachedFrontloader:getAttachedTrailersFillLevelAndCapacity();
			if f ~= nil and c ~= nil then
				fillLevel = fillLevel + f;
				capacity = capacity + c;
				hasTrailer = true;
			end;
		end;		
	end;	
    if hasTrailer then
        return fillLevel, capacity;
    end;
    return nil;
end;
